function prob = find_exit_probability(T,V,F_n,sj,sk,D_A,D_B,D_b,control,time_step)
%from si we want to go to sj, when previous state was sk (sk~=sj)

% V=T.Vert{si};
H=cddmex('hull',struct('V',V));    %H-repres of polytope (facets are given by H.A x - H.B = 0)
f_A=H.A; %facets of current polytope
f_B=H.B;

% prec=1e-9;
prec=0;
prec2=1e-9;
max_steps=10000;

%find subsegment of exit facet where we have exiting points (positive projections on exit normal) - set delimited by v1 and v2
[v1,v1_ind]=setdiff(V,T.Vert{sk},'rows');  %vertex of si that is not in sk; it's index is also index of facet between si and sk
[ignore,ex_ind]=setdiff(V,T.Vert{sj},'rows');  %vertex of si that is not in sj; it's index is also index of facet between si and sj
v2_ind=setdiff(1:3,[v1_ind,ex_ind]);  %index of second vertex of common facet between si and sj
v2=V(v2_ind,:);
v1=v1'; %column vectors
v2=v2';
v3=V(ex_ind,:)';
init_vert=[v1,v2,v3];   %initial vertices (some values v1,v2,v3 might be altered

%v1, v2 vertices of exit facet (between si and sj), v1_ind index of entry facet (between si and sk); vertices of entry facet are v2 and v3
%F_n(v1_ind,:) is the outer normal of entry facet
%F_n(ex_ind,:) is the outer normal of exit facet to sj

vf_val_v1=D_A*v1+D_B*control*[v1;1]+D_b;    %vector field value at v1
vf_val_v2=D_A*v2+D_B*control*[v2;1]+D_b;    %vector field value at v2
vf_val_v3=D_A*v3+D_B*control*[v3;1]+D_b;    %vector field value at v3

if (F_n(v1_ind,:)*vf_val_v2>=0) && (F_n(v1_ind,:)*vf_val_v3>=0)  %cannt exit to sj (after entry facet is hit, trajectories bounce back to sk)
    %     disp('Cannot exit to sj')
    prob=0;
    return
elseif (F_n(v1_ind,:)*vf_val_v2<-prec) && (F_n(v1_ind,:)*vf_val_v3>=0)  %v2 is good, move v3
    syms lambda;
    P=lambda*v2+(1-lambda)*v3;  %symbolic point of entry facet
    vf_val=F_n(v1_ind,:)*(D_A*P+D_B*control*[P;1]+D_b); %vector field value at P, projected to entry outer normal
    lambda=double(solve(vf_val,lambda));    %a single solution (in interval [0,1])
    v3=lambda*v2+(1-lambda)*v3; %v3 is moved here
elseif (F_n(v1_ind,:)*vf_val_v2>=0) && (F_n(v1_ind,:)*vf_val_v3<-prec)  %v3 is good, move v2
    syms lambda;
    P=lambda*v2+(1-lambda)*v3;  %symbolic point of entry facet
    vf_val=F_n(v1_ind,:)*(D_A*P+D_B*control*[P;1]+D_b); %vector field value at P, projected to entry outer normal
    lambda=double(solve(vf_val,lambda));    %a single solution (in interval [0,1])
    v2=lambda*v2+(1-lambda)*v3; %v2 is moved here    
else    %both v2 and v3 are good, nothing to do here

end

%integrate forward from v2
if isequal(v2,init_vert(:,2)) && (F_n(v1_ind,:)*vf_val_v2>0)   %v2 exits back through entry facet
    prob=0;
    return
end
%integrate from v2 (obtain point P2 and exit facet exit_v2)
if isequal(v2,init_vert(:,2)) && (F_n(ex_ind,:)*vf_val_v2>0)   %v2 leaves immediately the polytope to sj
    P2=v2;
    exit_v2=ex_ind;
else    %integrate forward from v2
    x=v2;  %current point (column vector with 2 elements)
    steps=0;
%     while inpolygon(x(1),x(2),V(:,1),V(:,2))
    while sum(H.A*x-H.B <=prec2)==3
        prev_x=x;   %we'll need last position in polytope and first outside
        x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
        x = prev_x + x_dot*time_step;  %next position (integrate forward)
        steps=steps+1;
        if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
            prob=0;
            return
        end
    end
    %triangle "si" was just left -> x is the first point outside, prev_x is the last one inside si
    %distance from x to facets of si:
    dist=zeros(1,3);
    x_mean=mean([x,prev_x],2);
    x0=x_mean(1); %coordinates of point we want to find to which facet is closer
    y0=x_mean(2);
    for i=1:3
        vert_ind=setdiff(1:3,i);   %vertices of facet i
        x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
        y1=V(vert_ind(1),2);
        x2=V(vert_ind(2),1);
        y2=V(vert_ind(2),2);
        %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
        dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
    end
    [ignore,exit_v2]=min(dist); %index of exit facet
    %find point on exit facet (as intersection of two lines, given by (prev_x,x) and facet ind
    %see formula at http://en.wikipedia.org/wiki/Line-line_intersection, http://mathworld.wolfram.com/Line-LineIntersection.html
    vert_ind=setdiff(1:3,exit_v2);   %vertices of exit facet when starting from v2
    x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
    y1=V(vert_ind(1),2);
    x2=V(vert_ind(2),1);
    y2=V(vert_ind(2),2);
    x3=prev_x(1);
    y3=prev_x(2);
    x4=x(1);
    y4=x(2);
    Px=((x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
    Py=((x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
    %just for testing
    if Px<min([x1,x2]) || Px>max([x1,x2]) || Py<min([y1,y2]) || Py>max([y1,y2])
        fprintf('\n Error in finding point on specific facet\n');
    end
    P2=[Px; Py];    %point P2 is on facet exit_v2
end

if exit_v2~=ex_ind  %didn't exit to sj from v2, so it's impossible from anywhere on entering facet
    if (exit_v2==v1_ind) && ~isequal(v2,init_vert(:,2)) %v2 was moved, and it exits (via integration) throught entry facet
        if (F_n(ex_ind,:)*vf_val_v2<-prec) && (F_n(ex_ind,:)*vf_val_v1>=0)  %move v2 on exit facet
            syms lambda;
            P=lambda*init_vert(:,2)+(1-lambda)*v1;  %symbolic point of entry facet
            vf_val=F_n(ex_ind,:)*(D_A*P+D_B*control*[P;1]+D_b); %vector field value at P, projected to entry outer normal
            lambda=double(solve(vf_val,lambda));    %a single solution (in interval [0,1])
            v2_t=lambda*init_vert(:,2)+(1-lambda)*v1; %v2 is moved here
%             plot(v2_t(1),v2_t(2),'*r')
            %integrate back from v2_t
            x=v2_t;  %current point (column vector with 2 elements)
            steps=0;
            % while inpolygon(x(1),x(2),V(:,1),V(:,2))
            while sum(H.A*x-H.B <=prec2)==3
                prev_x=x;   %we'll need last position in polytope and first outside
                x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
                x = prev_x - x_dot*time_step;  %previous position (integrate backward)
                steps=steps+1;
                if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
                    prob=0;
                    return
                end
            end
            %triangle "si" was just left (by moving backwards) -> x is the first point outside, prev_x is the last one inside si
            %distance from x to facets of si:
            dist=zeros(1,3);
            x_mean=mean([x,prev_x],2);
            x0=x_mean(1); %coordinates of point we want to find to which facet is closer
            y0=x_mean(2);
            for i=1:3
                vert_ind=setdiff(1:3,i);   %vertices of facet i
                x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
                y1=V(vert_ind(1),2);
                x2=V(vert_ind(2),1);
                y2=V(vert_ind(2),2);
                %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
                dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
            end
            [ignore,init_v2_t]=min(dist); %index of exit facet
            %find point on exit facet (as intersection of two lines, given by (prev_x,x) and facet ind
            %see formula at http://en.wikipedia.org/wiki/Line-line_intersection, http://mathworld.wolfram.com/Line-LineIntersection.html
            vert_ind=setdiff(1:3,init_v2_t);   %vertices of initial facet for exting through point v1
            x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
            y1=V(vert_ind(1),2);
            x2=V(vert_ind(2),1);
            y2=V(vert_ind(2),2);
            x3=prev_x(1);
            y3=prev_x(2);
            x4=x(1);
            y4=x(2);
            Px=((x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
            Py=((x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
            %just for testing
            % if Px<min([x1,x2]) || Px>max([x1,x2]) || Py<min([y1,y2]) || Py>max([y1,y2])
            %     fprintf('\n Error in finding initial point on specific facet by backwards integration!\n');
            %     plot(Px,Py,'b*')
            %     [h_exit_vvf] = vect_field_vertex(V, control,D_A,D_B,D_b);
            %     pause
            % end
            P1=[Px; Py];    %point P1 is on facet init_v2_t, and starting from here we exit through point v2_t
            if init_v2_t==v1_ind  %v2_t is reached from a point P1 on entry facet
                v2=P1;  %update v2 to this point, use it from now on
            else
                prob=0;
                return
            end
        else
            prob=0;
            return
        end
    else
        prob=0;
        return
    end
end

%v2 exits to sj; integrate forward from v3
if isequal(v3,init_vert(:,3)) && (F_n(v1_ind,:)*vf_val_v3>0)   %v3 exits back through entry facet (F1)
    prob=0;
    return
end
%integrate from v3 (obtain point P3 and exit facet exit_v3)
if isequal(v3,init_vert(:,3)) && (F_n(v2_ind,:)*vf_val_v3>0)   %v3 leaves immediately the polytope through F2 (the other facet, not enter, not exit)
    P3=v3;
    exit_v3=v2_ind;
else    %integrate forward from v3
    x=v3;  %current point (column vector with 2 elements)
    steps=0;
%     while inpolygon(x(1),x(2),V(:,1),V(:,2))
    while sum(H.A*x-H.B <=prec2)==3
        prev_x=x;   %we'll need last position in polytope and first outside
        x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
        x = prev_x + x_dot*time_step;  %next position (integrate forward)
        steps=steps+1;
        if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
            prob=0;
            return
        end
    end
    %triangle "si" was just left -> x is the first point outside, prev_x is the last one inside si
    %distance from x to facets of si:
    dist=zeros(1,3);
    x_mean=mean([x,prev_x],2);
    x0=x_mean(1); %coordinates of point we want to find to which facet is closer
    y0=x_mean(2);
    for i=1:3
        vert_ind=setdiff(1:3,i);   %vertices of facet i
        x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
        y1=V(vert_ind(1),2);
        x2=V(vert_ind(2),1);
        y2=V(vert_ind(2),2);
        %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
        dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
    end
    [ignore,exit_v3]=min(dist); %index of exit facet
    %find point on exit facet (as intersection of two lines, given by (prev_x,x) and facet ind
    %see formula at http://en.wikipedia.org/wiki/Line-line_intersection, http://mathworld.wolfram.com/Line-LineIntersection.html
    vert_ind=setdiff(1:3,exit_v3);   %vertices of exit facet when starting from v2
    x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
    y1=V(vert_ind(1),2);
    x2=V(vert_ind(2),1);
    y2=V(vert_ind(2),2);
    x3=prev_x(1);
    y3=prev_x(2);
    x4=x(1);
    y4=x(2);
    Px=((x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
    Py=((x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
    %just for testing
    if Px<min([x1,x2]) || Px>max([x1,x2]) || Py<min([y1,y2]) || Py>max([y1,y2])
        fprintf('\n Error in finding point on specific facet\n');
    end
    P3=[Px; Py];    %point P3 is on facet exit_v3
end
%exit_v3 can be either exit facet, or the other facet (it's not entry facet, because this case was already tested)
if exit_v3==ex_ind  %exit_v3 is F3 (so is exit_v2)
    prob=norm(v2-v3)/norm(init_vert(:,2)-init_vert(:,3));   %ratio between legth from v2 to v3 and length of entry facet F1 (v2 and v3 may be moved)
    return
end

%exit_v3 is the other facet (F2); we have to integrate backwards from v1
%check if v1 has to be moved
if (F_n(ex_ind,:)*vf_val_v1<-prec)  %v1 is not good, have to move it on exiting facet
    if (F_n(ex_ind,:)*vf_val_v2>=0)  
        syms lambda;
        P=lambda*v1+(1-lambda)*init_vert(:,2);  %symbolic point of exit facet
        vf_val=F_n(ex_ind,:)*(D_A*P+D_B*control*[P;1]+D_b); %vector field value at P
        lambda=double(solve(vf_val,lambda));    %a single solution (in interval [0,1])
        v1=lambda*v1+(1-lambda)*init_vert(:,2); %v1 is moved here
    else    %both v1 and v2 have vector field pointing opposite of sj, so cannot exit through sj
        %disp('Cannot exit to sj')
        prob=0;
        return
    end
end %on else, v1 remains the same

%integrate backwards from v1 (obtain point P1 and entry facet init_v1)
if isequal(v1,init_vert(:,1)) && (F_n(ex_ind,:)*vf_val_v1<-prec)   %v1 leaves (backwards) immediately the polytope through F3 (the exit facet)
    P1=v1;
    init_v1=ex_ind; 
elseif isequal(v1,init_vert(:,1)) && (F_n(v2_ind,:)*vf_val_v1<-prec)   %v1 leaves (backwards) immediately the polytope through F2 (the other facet, not enter, not exit)
    P1=v1;
    init_v1=v2_ind;
    %in this case, we need to integrate backward&forward from a point on F2 (conditions after the current if)
else    %integrate forward from v3
    x=v1;  %current point (column vector with 2 elements)
    steps=0;
    % while inpolygon(x(1),x(2),V(:,1),V(:,2))
    while sum(H.A*x-H.B <=prec2)==3
        prev_x=x;   %we'll need last position in polytope and first outside
        x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
        x = prev_x - x_dot*time_step;  %previous position (integrate backward)
        steps=steps+1;
        if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
            prob=0;
            return
        end
    end
    %triangle "si" was just left (by moving backwards) -> x is the first point outside, prev_x is the last one inside si
    %distance from x to facets of si:
    dist=zeros(1,3);
    x_mean=mean([x,prev_x],2);
    x0=x_mean(1); %coordinates of point we want to find to which facet is closer
    y0=x_mean(2);
    for i=1:3
        vert_ind=setdiff(1:3,i);   %vertices of facet i
        x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
        y1=V(vert_ind(1),2);
        x2=V(vert_ind(2),1);
        y2=V(vert_ind(2),2);
        %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
        dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
    end
    [ignore,init_v1]=min(dist); %index of exit facet
    %find point on exit facet (as intersection of two lines, given by (prev_x,x) and facet ind
    %see formula at http://en.wikipedia.org/wiki/Line-line_intersection, http://mathworld.wolfram.com/Line-LineIntersection.html
    vert_ind=setdiff(1:3,init_v1);   %vertices of initial facet for exting through point v1
    x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
    y1=V(vert_ind(1),2);
    x2=V(vert_ind(2),1);
    y2=V(vert_ind(2),2);
    x3=prev_x(1);
    y3=prev_x(2);
    x4=x(1);
    y4=x(2);
    Px=((x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
    Py=((x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
    %just for testing
    % if Px<min([x1,x2]) || Px>max([x1,x2]) || Py<min([y1,y2]) || Py>max([y1,y2])
    %     fprintf('\n Error in finding initial point on specific facet by backwards integration!\n');
    %     plot(Px,Py,'b*')
    %     [h_exit_vvf] = vect_field_vertex(V, control,D_A,D_B,D_b);
    %     pause
    % end
    P1=[Px; Py];    %point P1 is on facet init_v1, and starting from here we exit through point v1
end

%we know that v2 exits through F3 (exit facet) and v3 through F2 (the other facet); v1 is reached by starting from point P1, which is on facet init_v1
if init_v1==v1_ind  %v1 is reached from a point P1 on entry facet
    prob=norm(v2-P1)/norm(init_vert(:,2)-init_vert(:,3));   %ratio between legth from v2 to P1 and length of entry facet F1
    return
elseif isequal(v1,init_vert(:,1)) && (F_n(v2_ind,:)*vf_val_v1<-prec) %v1 reached directly from F2 (case from line 201-204) - find a point on F2 and integrate backward and forward from there
    v3=init_vert(:,3);
    if (F_n(v2_ind,:)*vf_val_v3>0)  %this and above line may be redundant: v3 exits directly through F2; v1 would enter directly from F2
        syms lambda;
        P=lambda*v3+(1-lambda)*v1;  %symbolic point of entry facet
        vf_val=F_n(v2_ind,:)*(D_A*P+D_B*control*[P;1]+D_b); %vector field value at P, projected to entry outer normal
        lambda=double(solve(vf_val,lambda));    %a single solution (in interval [0,1])
        v_F2=lambda*v3+(1-lambda)*v1; %pick point on F2 with zero proj on outernormal (v1 would be moved here)
        
        %integrate backwards from v_F2
%         lambda=lambda*1.01; %move a little towards v3, to avoid small errors
%         v_F2=lambda*v3+(1-lambda)*v1; %pick point on F2 with zero proj on outernormal (v1 would be moved here)
        x=v_F2;  %current point (column vector with 2 elements)
        steps=0;
        % while inpolygon(x(1),x(2),V(:,1),V(:,2))
        while sum(H.A*x-H.B <=prec2)==3
            prev_x=x;   %we'll need last position in polytope and first outside
            x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
            x = prev_x - x_dot*time_step;  %previous position (integrate backwards)
            steps=steps+1;
            if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
                init_vF2=v2_ind;    %abort integration; will exit if loop and use the leftover
                break;
            end
        end
        %triangle "si" was just left (by moving backwards) -> x is the first point outside, prev_x is the last one inside si
        %distance from x to facets of si:
        dist=zeros(1,3);
        x_mean=mean([x,prev_x],2);
        x0=x_mean(1); %coordinates of point we want to find to which facet is closer
        y0=x_mean(2);
        for i=1:3
            vert_ind=setdiff(1:3,i);   %vertices of facet i
            x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
            y1=V(vert_ind(1),2);
            x2=V(vert_ind(2),1);
            y2=V(vert_ind(2),2);
            %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
            dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
        end
        [ignore,init_vF2]=min(dist); %index of exit facet
        %find point on exit facet (as intersection of two lines, given by (prev_x,x) and facet ind
        %see formula at http://en.wikipedia.org/wiki/Line-line_intersection, http://mathworld.wolfram.com/Line-LineIntersection.html
        vert_ind=setdiff(1:3,init_vF2);   %vertices of initial facet for exting through point v1
        x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
        y1=V(vert_ind(1),2);
        x2=V(vert_ind(2),1);
        y2=V(vert_ind(2),2);
        x3=prev_x(1);
        y3=prev_x(2);
        x4=x(1);
        y4=x(2);
        Px=((x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
        Py=((x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
        %just for testing
        % if Px<min([x1,x2]) || Px>max([x1,x2]) || Py<min([y1,y2]) || Py>max([y1,y2])
        %     fprintf('\n Error in finding initial point on specific facet by backwards integration!\n');
        %     plot(Px,Py,'b*')
        %     [h_exit_vvf] = vect_field_vertex(V, control,D_A,D_B,D_b);
        %     pause
        % end
        Pin_vF2=[Px; Py];

        %integrate forward from v_F2 if backward integr. yielded F1 (entry facet)
        if init_vF2==v1_ind
%             lambda=lambda*0.99; %move a little towards v1, to avoid small errors
%             v_F2=lambda*v3+(1-lambda)*v1; %pick point on F2 with zero proj on outernormal (v1 would be moved here)
            x=v_F2;  %current point (column vector with 2 elements)
            steps=0;
            %     while inpolygon(x(1),x(2),V(:,1),V(:,2))
            while sum(H.A*x-H.B <=prec2)==3
                prev_x=x;   %we'll need last position in polytope and first outside
                x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
                x = prev_x + x_dot*time_step;  %next position (integrate forward)
                steps=steps+1;
                if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
                    exit_vF2=v2_ind;    %abort integration; will exit if loop and use the leftover
                    break;
                end
            end
            %triangle "si" was just left -> x is the first point outside, prev_x is the last one inside si
            %distance from x to facets of si:
            dist=zeros(1,3);
            x_mean=mean([x,prev_x],2);
            x0=x_mean(1); %coordinates of point we want to find to which facet is closer
            y0=x_mean(2);
            for i=1:3
                vert_ind=setdiff(1:3,i);   %vertices of facet i
                x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
                y1=V(vert_ind(1),2);
                x2=V(vert_ind(2),1);
                y2=V(vert_ind(2),2);
                %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
                dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
            end
            [ignore,exit_vF2]=min(dist); %index of exit facet
            %find point on exit facet (as intersection of two lines, given by (prev_x,x) and facet ind
            %see formula at http://en.wikipedia.org/wiki/Line-line_intersection, http://mathworld.wolfram.com/Line-LineIntersection.html
            vert_ind=setdiff(1:3,exit_vF2);   %vertices of exit facet when starting from v2
            x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
            y1=V(vert_ind(1),2);
            x2=V(vert_ind(2),1);
            y2=V(vert_ind(2),2);
            x3=prev_x(1);
            y3=prev_x(2);
            x4=x(1);
            y4=x(2);
            Px=((x1*y2-y1*x2)*(x3-x4)-(x1-x2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
            Py=((x1*y2-y1*x2)*(y3-y4)-(y1-y2)*(x3*y4-y3*x4))/((x1-x2)*(y3-y4)-(y1-y2)*(x3-x4));
            %just for testing
            if Px<min([x1,x2]) || Px>max([x1,x2]) || Py<min([y1,y2]) || Py>max([y1,y2])
                fprintf('\n Error in finding point on specific facet\n');
            end
            Pex_vF2=[Px; Py];    %point P3 is on facet exit_v3

            if exit_vF2==ex_ind %integration backward and forward from vF2 is good (vF2 is the point where a traj in tangent to F2 and after that leaves through F3)
                prob=norm(v2-Pin_vF2)/norm(init_vert(:,2)-init_vert(:,3));   %ratio between legth from v2 to v, and length of entry facet F1
%                 plot(v_F2(1),v_F2(2),'*b',Pin_vF2(1),Pin_vF2(2),'*g',Pex_vF2(1),Pex_vF2(2),'*k');
                return
            end
        end
    end
end

%if function not yet returned, try conservative test (v1 is reached from F2 or F3)
%try forward integration from middle point between v2 and v3 (or we can pick several random points)
v=mean([v2,v3],2);  %current point (column vector with 2 elements)
x=v;
steps=0;
%     while inpolygon(x(1),x(2),V(:,1),V(:,2))
while sum(H.A*x-H.B <=prec2)==3
    prev_x=x;   %we'll need last position in polytope and first outside
    x_dot=D_A*x+D_B*control*[x;1]+D_b;  %resulting speed at x
    x = prev_x + x_dot*time_step;  %next position (integrate forward)
    steps=steps+1;
    if steps>max_steps  %a point very close to equilibrium exists and integration cannot advance
        prob=0;
        return
    end
end
%triangle "si" was just left -> x is the first point outside, prev_x is the last one inside si
%distance from x to facets of si:
dist=zeros(1,3);
x_mean=mean([x,prev_x],2);
x0=x_mean(1); %coordinates of point we want to find to which facet is closer
y0=x_mean(2);
for i=1:3
    vert_ind=setdiff(1:3,i);   %vertices of facet i
    x1=V(vert_ind(1),1);    %facet i is given by points (x1,y1),(x2,y2)
    y1=V(vert_ind(1),2);
    x2=V(vert_ind(2),1);
    y2=V(vert_ind(2),2);
    %see formula for distance at "http://mathworld.wolfram.com/Point-LineDistance2-Dimensional.html"
    dist(i)=abs((x2-x1)*(y1-y0)-(x1-x0)*(y2-y1))/sqrt((x2-x1)^2+(y2-y1)^2);
end
[ignore,exit_v]=min(dist); %index of exit facet
if exit_v==ex_ind  %correct exit
    prob=norm(v2-v)/norm(init_vert(:,2)-init_vert(:,3));   %ratio between legth from v2 to v, and length of entry facet F1
    return
else
    %disp('Cannot exit to sj')
    prob=0;    %conservative
    return
end
