function [sigma,paths,prob_paths] = find_path_PN_state_machine(Pre,Post,prob_trans,places,initial_state,final_state,trans_strings,places_strings)
% construct a path with maximum probility assuming one robot. 
% Pre and Post define the Petri net model
% prob_trans is the probability vector of firing of each transition
% initial_state is a 2x1 matrix where the first element is the initial
% region of the robot while the second one is the initial previous region
% final_state is a scalar indicating the desired final region for the robot 
% trans_strings and places_strings are used to print some messages

%create the vector of initial marking m0
robot = {}; %keep the places in which we want to send the robots
m0 = zeros(size(Pre,1),1); %create the vector containing the initial marking
for i = 1 : size(initial_state,1)
    for j = 1 : size(places.tab_places,1)
        if (sum(places.tab_places(j,:) == initial_state(i,:)) == 2)
            m0(j) = m0(j)+1;
            robot(i).initial = j;
            break;
        end        
    end
end

nplaces = size(Pre,1); %number of places
ntrans = size(Pre,2); %number of transitions

prob_log = -log(prob_trans)*10000+ones(1,ntrans); %take the logaritm of the probabilities and add the one vector in order to minimize also the number of transitions to fire 

index = find(prob_log==Inf);%replace Inf with a big value to avoid numerical problems
for i = 1 : length(index)
    prob_log(index(i)) = intmax;
end

C = Post - Pre; % incidence matrix
vartype=''; %string containing the variable types. for PN marking will be integer, i.e., I, and binar for the firing vector
ctype=''; % vector containg the type of the constraints 

integer_var=''; 
type_eq='';
type_U='';
for i = 1 : nplaces
    integer_var = sprintf('%sC',integer_var);
    type_eq = sprintf('%sS',type_eq);
    type_U=sprintf('%sU',type_U);
end
integer_var_trans='';
for i = 1 : ntrans
    integer_var_trans = sprintf('%sC',integer_var_trans);
end

%variables are: m si 

%create the constraint matrix A

%fundamental equation: m = m0 + C \cdot \sigma
A = [eye(nplaces) -C];
B = m0;
vartype = sprintf('%s%s%s',vartype,integer_var,integer_var_trans);
ctype = sprintf('%s%s',ctype,type_eq);

obj = [zeros(1,nplaces) prob_log];
    
new_final_state = unique(final_state);
for j = 1 : length(new_final_state)
    temp = zeros(1,size(A,2));
    for k = 1 : size(places.tab_places,1)
        if (places.tab_places(k,1) == new_final_state(j))
            temp( k ) = temp( k ) + 1;
        end
    end
    A = [A ; temp];
    B = [B ; length(find(final_state==new_final_state(j)))];
    ctype = sprintf('%sS',ctype);
end

[xopt, fmin, status, extra] = glpk(obj,A,B,zeros(1,size(A,2)),[],ctype,vartype,1);

if (status ~= 5)
    error('Error solving ILP. Error code %d',status);
end

sigma = round(xopt(nplaces+1:end));
m_final = find(round(xopt(1:nplaces))>0);
init = 1;
for i = 1 : length(m_final)
    for j = 1 : round(xopt(m_final(i)))
        robot(init).final = m_final(i);
        init = init + 1;
    end
end

prob_log = (prob_log - ones(1,ntrans))/10000;

fprintf(1,'\n\n\n');
marking = m0;
for i = 1 : length(robot)
    now = robot(i).initial;
    probab = 1;
    to_obtain_path = [initial_state(i,2) initial_state(i,1)];
    fprintf(1,'********************************************\n');
    fprintf(1,'*******Path for robot %d ***********\n',i);
    fprintf(1,'%s\n',places_strings{now});
    while 1
        arrived = 0;
        for j = 1 : length(robot)
            if ~isempty(intersect(now,robot(j).final));
                if (probab > eps)
                    fprintf(1,'Robot %d reaches a final region.\n',i);
                    fprintf(1,'Probability of the path: %s.\n',num2str(probab));
                    prob_paths(i) = probab;
                else
                    fprintf(1,'This path cannot be followed having null probability.\n');
                    prob_paths(i) = 0;
                end
                paths{i} = to_obtain_path;
                fprintf(1,'********************************************\n');
                marking(now) = marking(now) - 1;
                arrived = 1;
                robot(j).final = [];
                break;
            end
        end
        if (arrived == 1)
            break;
        end
        trans = intersect(find(Pre(now,:)),find(sigma)); %find enabled transitions from place now
        fire = trans(ceil(length(trans)*rand)); %fire random an enabled transition
        marking = marking + C(:,fire);
        probab = probab * prob_trans(fire);
        sigma(fire) = sigma(fire) - 1;
        fprintf(1,'%s\n',trans_strings{fire});
        now = find(Post(:,fire));
        to_obtain_path = [to_obtain_path places.regions(now)];
%        fprintf(1,'%s\n',places_strings{now});
    end
end

    

