function [X,goal_reached] = simulate_cont_traj(T,Ctrl,path_T,ctrl_seq_T,D_A,D_B,D_b,time_step,x0,direction)
%integrate from x0 until path_T is left, or goal state (simplex) is reached
%direction is 1 for forward integration (and -1 for backward integration)
%special cases (integrating from a vertex and immediately leave the simplex are not handled
%path_T is the sequence of simplices, ctrl_seq is the sequence of controllers to be applied (linear indices in Ctrl)

prec=1e-2;
max_steps=10000;
X(:,1)=x0;  %current point (column vector with 2 elements)

for i=1:length(path_T)-1    %don;t have to go to last state, because ctrl_seq is shorter than path_T
    si=path_T(i); %current simplex
    V=T.Vert{si};
    H=cddmex('hull',struct('V',V));    %H-repres of polytope (facets are given by H.A x - H.B = 0)
    
    %current continuous state is X(:,end); integrate until triangle "si" is left
    if sum(H.A*X(:,end)-H.B <=prec)==3  %current position is in simplex "si", integrate
        steps=0;
        % while inpolygon(x(1),x(2),V(:,1),V(:,2))
        while sum(H.A*X(:,end)-H.B <=prec)==3  %still in simplex
            x_dot=D_A*X(:,end)+D_B*Ctrl{ctrl_seq_T(i)}*[X(:,end);1]+D_b;  %resulting speed at current position x, given the current control from Ctrl
            X(:,end+1) = X(:,end) + direction*x_dot*time_step;  %next position (sign +/- for forward/backward integration is given by direction)
            steps=steps+1;
            if steps>max_steps  %a point very close to equilibrium exists in current simplex and integration cannot advance
                goal_reached=0;
                return
            end
        end
        %triangle "si" was just left -> X(:,end) is the first point outside, X(:,end-1) is the last one inside si
        
    else    %current posiiton is not on the desired path anymore, report failure in following path
        goal_reached=0;
        return
    end
end

%check if current position is in the last triangle from path
si=path_T(end); %last simplex
V=T.Vert{si};
H=cddmex('hull',struct('V',V));    %H-repres of polytope (facets are given by H.A x - H.B = 0)
if sum(H.A*X(:,end)-H.B <=prec)==3  %current continuous state (X(:,end)) is in simplex "si", report success
    goal_reached=1;
else
    goal_reached=0;
end
