function [C,adj,varargout]=triangular_decomposition(objects,x_max,y_max)

%first begin creating points (vertices) X
X = [0 0; %space boundary
    x_max 0;
    x_max y_max;
    0 y_max];

%begin creating constraints Cst (indices of lines from X giving linear constraints)
Cst = [1 2; %space boundary
         2 3;
         3 4;
         4 1];

%add obstacle information
for i=1:length(objects)
    v_O{i}=[];  %indices in X of vertices of each obstacle (will be used for finding unfeasible triangles)
    ind=size(X,1)+1;    %index for points
    X=[ X; [objects{i}]' ];
    for j=1:(size(objects{i},2)-1)
        Cst=[ Cst; [ind+j-1 , ind+j ] ];  %object's vertices are arranged in order of parcurging convex hull
        v_O{i}=[v_O{i}, ind+j-1];
    end
    Cst=[ Cst; [ind+j , ind ] ];  %close current object
    v_O{i}=[v_O{i}, ind+j];
end

%constrained triangulation (Matlab 2010b)
triangles = DelaunayTri(X, Cst);
triangles=triangles.Triangulation;  %don't need the other data from returned structure triangles

%from the returned triangles, some are inside obstacles (obstacles are triangulated as well); cannot have triangles partially overlapping with obstacles
%search indices of traingles from free space
ind=[]; %feasible indices of triangles (row indices in triangles)
for k=1:size(triangles,1)
    ind=[ind,k];    %assume k is good (if not, it will be removed)
    for i=1:length(objects) %for each obstacle
        if sum(ismember(triangles(k,:),v_O{i}))==3      %current triangle is inside obstacle i (current traingle has 3 vertices, all belong to the same obstacle)
            ind(end)=[];    %remove k from feasible indices
            break;  %continue with next index k
        end
    end
end

%construct cell C (triangle vertices in each element) and adjacency adj
%return: C- containing cel vertices (cell with each element a matrix with 2 rows and 3 columns)
%    and adj - adjacency matrix, with adj(i,j)=1 if cell i is adjacent to j (adj(i,i)=1 and adj is symmetric)

k=length(ind);  %number of feasible triangles

C=cell(1,k);
adj=sparse(eye(k)); %self transitions in every cell

if nargout>2    %if desired, compute middle points between adjacent cells (useful for an angular path finding); avoid reutrn a cell, because of memory usage:
    middle_X=sparse(k,k);   %element (i,j) is zero if i=j or if cells i and j are not adjacent, and otherwise it contains X-coordinate for middle segment between i and j
    middle_Y=sparse(k,k);
end

for i=1:k
    C{i}=(X(triangles(ind(i),:),:))';    %in matrix triangle, the columns are indices of points from X composing the triangle
    for j=i+1:k %go only through higher indices (adjacency matrix is symmetric)
        common_v=intersect(triangles(ind(i),:), triangles(ind(j),:)); %indices of common vertices (nodes) of cells i and j
        if length(common_v)==2 %two common vertices means adjacency
            adj(i,j)=1;
            adj(j,i)=1; %symmetric
            
            if nargout>2
                %middle of segment between cells i and j:
                middle_temp=mean(X(common_v,:));
                middle_X(i,j)=middle_temp(1);
                middle_X(j,i)=middle_temp(1);
                middle_Y(i,j)=middle_temp(2);
                middle_Y(j,i)=middle_temp(2);
            end
        end
    end
end
% adj=sparse(adj);    %convert to sparse

%arrange vertices of each cell from decomposition to be convex hull
for i=1:length(C)
    ch_in=convhull(C{i}(1,:),C{i}(2,:));
    C{i}=C{i}(:,ch_in(1:length(ch_in)-1));
end

if nargout>2
    %return middle of adjacent segments:
    varargout(1)={middle_X};
    varargout(2)={middle_Y};
end
