% clear all;close all;clc
addpath('functions','cdd','glpkmex');

%change environment bounds and dynamics on the following lines
x_max=10;   %environment bounds
y_max=10;

%affine dynamics:\dot{x}=D_A*x+D_B*u+D_b;
D_A=[ 0.5, -0.6; 0.7, -0.4];
D_B=eye(2);
D_b=[0.3, -0.3]';

U=[];   %control set
U_dim=3;    %radius of circle bounding controls
ang=linspace(0,2*pi,1+12);  %approx. witha 12-side polygon
ang=ang(1:end-1);
for i=1:length(ang)
    U=[U; U_dim*cos(ang(i)) U_dim*sin(ang(i))];
end

% regions = define_regions(x_max,y_max);    %define obstacles via mouse clicks
load obstacles.mat  %load obstacles from file
% regs_h = plot_environment2(regions,x_max,y_max);


T = create_abstraction(regions,x_max,y_max);  %create triangular partition (obstacles not covered by triangles)
env_h = plot_environment2(regions,x_max,y_max,T.Vert);
plot_vect_field([0 0;x_max 0;x_max y_max; 0 y_max], zeros(2,3) ,D_A,D_B,D_b,30);    %v.f. of uncontrolled system
set(gca,'Box','on')
% axis('square')
% set(gca,'Xtick',[0,10],'Ytick',[0,10])

[Ctrl,Prob] = find_controls_probabilities(T,D_A,D_B,D_b,U); %find controls and history-based probabilities

[Pre,Post,prob_trans,places,trans_strings,places_strings]= construct_PN2(T,Prob);

if (is_strongly_connected(Pre,Post) == 1)
    fprintf(1,'\nThe Petri net is a strongly connected state-machine\n');
end


% from_file = input('Load initial/final regions of robots from the main file (y/n)=','s');
from_file='y';    %define initial&final states in this file (two lines below)
if strcmpi(from_file,'y')
    initial_state=[24 14; 23 24]; %just an example for two robots
    final_state=[4 30];
else
    r = input('Number of robots=');
    initial_state = zeros(r,2);
    final_state = zeros(r,1);
    for i = 1 : r
        fprintf(1,'=======================================================\nRobot %d\n',i);
        initial_state(i,1) = input('Initial region =');
        initial_state(i,2) = input('The region from which the robot entered in the initial region =');
        final_state(i) = input('Final region =');
    end
end


%create initial marking vector
m0 = zeros(size(Pre,1),1); %create the vector containing the initial marking
for i = 1 : size(initial_state,1)
    for j = 1 : size(places.tab_places,1)
        if (sum(places.tab_places(j,:) == initial_state(i,:)) == 2)
            m0(j) = m0(j)+1;
            break;
        end
    end
end

[sigma,paths,prob_paths] = find_path_PN_state_machine(Pre,Post,prob_trans,places,initial_state,final_state,trans_strings,places_strings);


%simulate continuous traj.
lambda=[0.2 0.5];  %where each initial point is located on starting facet
colors=['r', 'g', 'b', 'm', 'c'];
time_step=1e-3;
env_h = plot_environment2(regions,x_max,y_max,T.Vert);
for i=1:length(paths)   %for each robot
    ctrl_seq{i} = ctrl_ind_path(paths{i},Prob);
    init_V=intersect(T.Vert{initial_state(i,1)},T.Vert{initial_state(i,2)},'rows')';	%starting facet
    x0 = lambda(i)*init_V(:,1) + (1-lambda(i))*init_V(:,2);
    [X{i},goal_reached(i)] = simulate_cont_traj(T,Ctrl,paths{i}(2:end),ctrl_seq{i},D_A,D_B,D_b,time_step,x0,1); %simulate evolution of system (integrate forward), until goal cell is reached, or path_T is left
    if goal_reached(i)==1
        fprintf('\nRobot %d follows path and reaches target state.\n',i)
    else
        fprintf('\nRobot %d did NOT reach target.\n',i)
    end

    plot(x0(1),x0(2),'*','Color',colors(i))
    plot(X{i}(1,:),X{i}(2,:),'Color',colors(i),'LineWidth',1.5)
end

rmpath('functions','cdd','glpkmex');
